@@ -23,6 +23,7 @@ import re
 import string
 import time
 import warnings
+import logging
 from collections import deque, OrderedDict
 
 import requests
@@ -32,7 +33,7 @@ from threading import Barrier, Condition, Thread
 
 from .multipart import Data
 
-__version__ = "0.9.2"
+__version__ = "0.9.5"
 
 BASE_URL = "https://volafile.io"
 BASE_ROOM_URL = BASE_URL + "/r/"
@@ -53,14 +54,6 @@ def to_json(obj):
     return json.dumps(obj, separators=(',', ':'))
 
 
-def verify_username(username):
-    """Raises an exception if the given username is not valid."""
-    if len(username) > 12 or len(username) < 3:
-        raise ValueError("Username must be between 3 and 12 characters.")
-    if any(c not in string.ascii_letters + string.digits for c in username):
-        raise ValueError("Usernames can only contain alphanumeric characters.")
-
-
 def parse_chat_message(data):
     """Parses the data for a json chat message and returns a
     ChatMessage object"""
@@ -159,14 +152,17 @@ class RoomConnection(Connection):
 
         self._ping_interval = 20  # default
 
-    def subscribe(self, room_name, username):
+    def subscribe(self, room_name, username, secret_key):
         """Make subscribe API call"""
         checksum, checksum2 = self._get_checksums(room_name)
-        obj = [-1, [[0, ["subscribe", {"room": room_name,
-                                       "checksum": checksum,
-                                       "checksum2": checksum2,
-                                       "nick": username
-                                       }]],
+        subscribe_options = {"room": room_name,
+                             "checksum": checksum,
+                             "checksum2": checksum2,
+                             "nick": username
+                             }
+        if secret_key:
+            subscribe_options['secretToken'] = secret_key
+        obj = [-1, [[0, ["subscribe", subscribe_options]],
                     0]]
         self.send_message("4" + to_json(obj))
 
@@ -180,8 +176,8 @@ class RoomConnection(Connection):
             cs1 = re.search(r'config\.checksum\s*=\s*"(\w+?)"', text).group(1)
 
             return cs1, cs2
-        except Exception as ex:
-            raise IOError("Failed to get checksums") from ex
+        except Exception:
+            raise IOError("Failed to get checksums")
 
     def enqueue_file(self, file_id):
         """Enqueues a file id so listeners can see the file."""
@@ -255,6 +251,8 @@ class RoomConnection(Connection):
                             with self.condition:
                                 self.condition.notify_all()
 
+            except:
+                logging.exception('')
             finally:
                 try:
                     self.close()
@@ -296,23 +294,46 @@ class Room:
 
         self.conn = RoomConnection()
 
+        room_resp = None
         self.name = name
         if not self.name:
-            name = self.conn.get(BASE_URL + "/new").url
+            room_resp = self.conn.get(BASE_URL + "/new")
+            url = room_resp.url
             try:
-                self.name = re.search(r'r/(.+?)$', name).group(1)
-            except Exception as ex:
-                raise IOError("Failed to create room") from ex
+                self.name = re.search(r'r/(.+?)$', url).group(1)
+            except Exception:
+                raise IOError("Failed to create room")
 
+        self._config = {}
         try:
-            text = self.conn.get(BASE_ROOM_URL + self.name).text
-            self.title = re.search(r'name\s*:\s*"(.+?)"', text).group(1)
-        except Exception as ex:
-            raise IOError("Failed to get room title") from ex
+            if not room_resp:
+                room_resp = self.conn.get(BASE_URL + "/" + self.name)
+            text = room_resp.text
+            text = text.replace('\n', '')
+            text = re.sub(r'(\w+):', r'"\1":', text)
+            text = text.replace('true', '"true"').replace('false', '"false"')
+            text = re.search(r'config=({.+});', text).group(1)
+            config = json.loads(text)
 
-        self.user = User(user, self.conn)
+            self._config['title'] = config['name']
+            self._config['private'] = config['private'] == 'true'
+            self._config['motd'] = config['motd']
+            secret_key = config.get('secretToken')
 
-        self.conn.subscribe(self.name, self.user.name)
+            self._config['max_title'] = config['max_room_name_length']
+            self._config['max_message'] = config['max_message_length']
+            max_nick = config['max_alias_length']
+            self._config['max_file'] = config['file_max_size']
+            self._config['ttl'] = config['file_time_to_live']
+            self._config['session_lifetime'] = config['session_lifetime']
+
+        except Exception:
+            raise IOError("Failed to get room config")
+
+        self.user = User(user, self.conn, max_nick)
+        self.owner = False
+
+        self.conn.subscribe(self.name, self.user.name, secret_key)
 
         self._user_count = 0
         self._files = OrderedDict()
@@ -380,13 +401,22 @@ class Room:
             elif data_type == "changed_config":
                 change = data
                 if change['key'] == 'name':
-                    self.title = change['value']
+                    self._config['title'] = change['value']
+                elif change['key'] == 'private':
+                    self._config['private'] = change['value'] == 'true'
+                elif change['key'] == 'motd':
+                    self._config['motd'] = change['value']
+                else:
+                    warnings.warn(
+                        "unknown config key '{}'".format(
+                            change['key']),
+                        Warning)
             elif data_type == "chat_name":
-                self.user.change_nick(data)
-            elif data_type == "time":
-                pass  # yup, that's the time. Thanks Laino
-            elif data_type == "subscribed":
-                pass  # fuck you Lain
+                self.user.name = data
+            elif data_type == "owner":
+                self.owner = True
+            elif data_type in ("time", "subscribed"):
+                pass
             else:
                 warnings.warn(
                     "unknown data type '{}'".format(data_type),
@@ -425,10 +455,18 @@ class Room:
     def post_chat(self, msg, me=False):
         """Posts a msg to this room's chat. Set me=True if you want to /me"""
         # pylint: disable=invalid-name
+        if len(msg) > self._config['max_message']:
+            raise ValueError(
+                "Chat message must be at most {} characters".format(
+                    self._config['max_message']))
+
+        while not self.user.name:
+            with self.conn.condition:
+                self.conn.condition.wait()
         if not me:
             self.conn.make_call("chat", [self.user.name, msg])
         else:
-            self.conn.make_call("command", [self.name, "me", msg])
+            self.conn.make_call("command", [self.user.name, "me", msg])
 
     def upload_file(self, filename, upload_as=None, blocksize=None,
                     callback=None):
@@ -467,27 +505,47 @@ class Room:
             self.conn.close()
 
     @property
-    def room_title(self):
+    def title(self):
         """Gets the title name of the room (e.g. /g/entoomen)"""
-        return self.title
+        return self._config['title']
 
-    def set_room_title(self, new_name):
-        """Sets the room name"""
-        if len(new_name) > 24 or len(new_name) < 1:
+    def set_title(self, new_name):
+        """Sets the room name (e.g. /g/entoomen)"""
+        if not self.owner:
+            raise RuntimeError("You must own this room to do that")
+        if len(new_name) > self._config['max_title'] or len(new_name) < 1:
             raise ValueError(
-                "Room name must be at most 24 characters and at least 1.")
+                "Room name length must be between 1 and {} characters.".format(
+                    self._config['max_title']))
         self.conn.make_call("editInfo", [{"name": new_name}])
+        self._config['title'] = new_name
 
-    def set_room_private(self, private):
+    @property
+    def private(self):
+        """True if the room is private, False otherwise"""
+        return self._config['private']
+
+    def set_private(self, value):
         """Sets the room to private if given True, else sets to public"""
-        private = "true" if private else "false"
-        self.conn.make_call("editInfo", [{"private": private}])
+        if not self.owner:
+            raise RuntimeError("You must own this room to do that")
+        priv = "true" if value else "false"
+        self.conn.make_call("editInfo", [{"private": priv}])
+        self._config['private'] = value
+
+    @property
+    def motd(self):
+        """Returns the message of the day for this room"""
+        return self._config['motd']
 
     def set_motd(self, motd):
         """Sets the room's MOTD"""
+        if not self.owner:
+            raise RuntimeError("You must own this room to do that")
         if len(motd) > 1000:
             raise ValueError("Room's MOTD must be at most 1000 characters")
         self.conn.make_call("editInfo", [{"motd": motd}])
+        self._config['motd'] = motd
 
     def clear(self):
         """Clears the cached information, if any"""
@@ -561,11 +619,12 @@ class User:
 
     """Used by Room. Currently not very useful by itself"""
 
-    def __init__(self, name, conn):
+    def __init__(self, name, conn, max_len):
+        self._max_length = max_len
         if name is None:
             self.name = ""
         else:
-            verify_username(name)
+            self.verify_username(name)
         self.name = name
         self.conn = conn
         self.logged_in = False
@@ -598,7 +657,7 @@ class User:
         Note: Must be logged out to change nick"""
         if self.logged_in:
             raise RuntimeError("User must be logged out")
-        verify_username(new_nick)
+        self.verify_username(new_nick)
 
         self.conn.make_call("command", [self.name, "nick", new_nick])
         self.name = new_nick
@@ -635,5 +694,17 @@ class User:
         if 'error' in json_resp:
             raise ValueError("Wrong password.")
 
+    def verify_username(self, username):
+        """Raises an exception if the given username is not valid."""
+        if len(username) > self._max_length or len(username) < 3:
+            raise ValueError(
+                "Username must be between 3 and {} characters.".format(
+                    self._max_length))
+        if any(
+                c not in string.ascii_letters +
+                string.digits for c in username):
+            raise ValueError(
+                "Usernames can only contain alphanumeric characters.")
+
     def __repr__(self):
         return "<User({}, {})>".format(self.name, self.logged_in)
